"""
Color maps come from colorbrewer2.org as an Excel file. I've converted the
Excel file to CSV and here I convert it to JSON for easy reading into Python.

"""

import sys
from csv import DictReader
from collections import OrderedDict
import json


def new_sub_map(row, sm_dict):
    num_colors = int(row['NumOfColors'])
    sm_dict[num_colors] = OrderedDict()
    sub_map = sm_dict[num_colors]

    sub_map['NumOfColors'] = num_colors
    sub_map['Type'] = row['Type']
    sub_map['Colors'] = [(int(row['R']), int(row['G']), int(row['B']))]

    return sub_map


def read_csv_to_dict():
    color_maps = OrderedDict()

    for scheme_type in ('Sequential', 'Diverging', 'Qualitative'):
        color_maps[scheme_type] = OrderedDict()

    with open('colorbrewer_all_schemes.csv', 'r') as csvf:
        csv = DictReader(csvf)

        for row in csv:
            if row['SchemeType']:
                # first row of a new color map block
                color_maps[row['SchemeType']][row['ColorName']] = OrderedDict()
                current_map = color_maps[row['SchemeType']][row['ColorName']]

                current_submap = new_sub_map(row, current_map)

            elif row['ColorName']:
                # first row of a new sub-map block
                current_submap = new_sub_map(row, current_map)

            elif not row['ColorName']:
                # continuation of a sub-map block
                current_submap['Colors'].append((int(row['R']),
                                                 int(row['G']),
                                                 int(row['B'])))

    return color_maps


def save_to_json(color_maps):
    with open('colorbrewer_all_schemes.json', 'w') as f:
        json.dump(color_maps, f, indent=1)


def main():
    color_maps = read_csv_to_dict()
    save_to_json(color_maps)


if __name__ == '__main__':
    sys.exit(main())



from .wesanderson import __doc__, _palettes, print_maps, get_map
from .wesanderson import WesAndersonMap as _WesAndersonMap


def _load_maps():
    maps = {}

    for k, v in _palettes.items():
        maps[k] = _WesAndersonMap(k, v['type'], v['colors'], v['url'])

    return maps
globals().update(_load_maps())



from __future__ import print_function
"""
Color palettes derived from http://wesandersonpalettes.tumblr.com/.

"""

import webbrowser

from ..brewer2mpl import _ColorMap


_tumblr_template = 'http://wesandersonpalettes.tumblr.com/post/{0}'

# Tumblr palettes in chronological order
_palettes = {
    'Chevalier': {
        'colors': [
            (53, 82, 67), (254, 202, 73), (201, 213, 213), (187, 162, 137)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79263620764/hotel-chevalier')
    },
    'Moonrise1': {
        'colors': [
            (114, 202, 221), (240, 165, 176), (140, 133, 54), (195, 180, 119),
            (250, 208, 99)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79263667140/sam-i-love-you-but-you-dont-know-what-youre')
    },
    'Mendl': {
        'colors': [
            (222, 141, 185), (184, 192, 246), (207, 147, 135), (92, 128, 204)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79348206200/mendls-heaven')
    },
    'Margot1': {
        'colors': [
            (137, 119, 18), (243, 194, 164), (246, 159, 151), (254, 214, 140),
            (98, 144, 117)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79348364517/margot-takes-a-bath')
    },
    'Cavalcanti': {
        'colors': [
            (209, 170, 0), (8, 50, 19), (146, 148, 96), (111, 152, 121),
            (132, 33, 17)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79348553036/castello-cavalcanti-how-can-i-help')
    },
    'Moonrise2': {
        'colors': [
            (102, 124, 116), (181, 106, 39), (194, 186, 124), (31, 25, 23)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79641731527/sam-why-do-you-always-use-binoculars-suzy-it')
    },
    'Margot2': {
        'colors': [
            (118, 139, 147), (188, 36, 15), (249, 236, 197), (212, 115, 41)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79641785036/margot-takes-a-break')
    },
    'Moonrise3': {
        'colors': [
            (242, 218, 82), (197, 157, 0), (203, 203, 201), (27, 30, 20)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79783357790/suzy-ive-always-wanted-to-be-an-orphan-most-of')
    },
    'GrandBudapest1': {
        'colors': [
            (238, 174, 101), (251, 79, 85), (72, 19, 19), (204, 95, 39)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79784389334/the-grand-budapest-hotel')
    },
    'Moonrise4': {
        'colors': [
            (123, 135, 97), (193, 166, 46), (79, 143, 107), (59, 69, 60),
            (159, 50, 8)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format('79956897654/coming-soon')
    },
    'Zissou': {
        'colors': [
            (0, 153, 230), (18, 37, 90), (242, 56, 20), (223, 183, 139),
            (182, 195, 197)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79956949771/steve-zissou-dont-point-that-gun-at-him-hes-an')
    },
    'Royal1': {
        'colors': [
            (121, 164, 58), (242, 214, 175), (94, 72, 41), (24, 20, 1)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '79957796915/royal-o-reilly-tenenbaum-1932-2001')
    },
    'Darjeeling1': {
        'colors': [
            (158, 151, 151), (194, 142, 0), (131, 102, 89), (156, 90, 51)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '80149649946/jack-i-wonder-if-the-three-of-us-wouldve-been')
    },
    'FantasticFox1': {
        'colors': [
            (249, 219, 32), (147, 75, 78), (66, 23, 13), (194, 121, 34),
            (226, 200, 167)
        ],
        'type': 'Qualitative',
        'url': _tumblr_template.format(
            '80149872170/mrs-fox-you-know-you-really-are-fantastic-mr')
    }
}

_map_names = {}
for k in _palettes:
    _map_names[k.lower()] = k


class WesAndersonMap(_ColorMap):
    """
    Representation of a color map with matplotlib compatible
    views of the map.

    Parameters
    ----------
    name : str
    map_type : str
    colors : list
        Colors as list of 0-255 RGB triplets.
    url : str
        URL on the web where this color map can be viewed.

    Attributes
    ----------
    name : str
    map_type : str
    number : int
        Number of colors in color map.
    colors : list
        Colors as list of 0-255 RGB triplets.
    hex_colors : list
    mpl_colors : list
    mpl_colormap : matplotlib LinearSegmentedColormap
    wap_url : str
        URL on the web where this color map can be viewed.

    """
    def __init__(self, name, map_type, colors, url):
        super(WesAndersonMap, self).__init__(name, map_type, colors)
        self.wap_url = url

    def wap(self):
        """
        View this color palette on the web.
        Will open a new tab in your web browser.

        """
        webbrowser.open_new_tab(self.wap_url)  # pragma: no cover


def print_maps():
    """
    Print a list of Wes Anderson palettes.

    """
    namelen = max(len(k) for k in _palettes)
    fmt = '{0:' + str(namelen + 4) + '}{1:16}{2:}'

    for k in sorted(_palettes.keys()):
        print(fmt.format(k, _palettes[k]['type'], len(_palettes[k]['colors'])))


def get_map(name, reverse=False):
    """
    Get a Wes Anderson palette by name.

    Parameters
    ----------
    name : str
        Name of map. Use brewer2mpl.wap.print_maps to see available names.
    reverse : bool, optional
        If True reverse colors from their default order.

    Returns
    -------
    palette : WesAndersonMap

    """
    name = _map_names[name.lower()]
    palette = _palettes[name]

    if reverse:
        name += '_r'
        palette['colors'] = list(reversed(palette['colors']))

    return WesAndersonMap(
        name, palette['type'], palette['colors'], palette['url'])



"""
brewer2mpl is a pure Python package for accessing
colorbrewer2.org color maps from Python. With brewer2mpl you can get the
raw RGB colors of all 165 colorbrewer2.org color maps.

"""
from .brewer2mpl import *
from . import wesanderson

__version__ = '1.4'



#!/usr/bin/env python
from distutils.core import setup

with open('README.rst', 'r') as f:
    long_description = f.read()

setup(name='brewer2mpl',
      version='1.4',
      description='Connect colorbrewer2.org color maps to Python and matplotlib',
      long_description=long_description,
      author='Matt Davis',
      author_email='jiffyclub@gmail.com',
      url='https://github.com/jiffyclub/brewer2mpl/wiki',
      packages=['brewer2mpl', 'brewer2mpl.wesanderson'],
      package_data={'brewer2mpl': ['data/colorbrewer*']},
      classifiers=['License :: OSI Approved :: MIT License',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Topic :: Scientific/Engineering :: Visualization'])
